@@ -6,7 +6,15 @@
 
 G_BEGIN_DECLS
 
-/* BOOLEAN:INT (gs-marshal.list:1) */
+/* BOOLEAN:VOID (gs-marshal.list:1) */
+extern void gs_marshal_BOOLEAN__VOID (GClosure     *closure,
+                                      GValue       *return_value,
+                                      guint         n_param_values,
+                                      const GValue *param_values,
+                                      gpointer      invocation_hint,
+                                      gpointer      marshal_data);
+
+/* BOOLEAN:INT (gs-marshal.list:2) */
 extern void gs_marshal_BOOLEAN__INT (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
@@ -14,7 +22,7 @@ extern void gs_marshal_BOOLEAN__INT (GClosure     *closure,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
 
-/* BOOLEAN:BOOLEAN (gs-marshal.list:2) */
+/* BOOLEAN:BOOLEAN (gs-marshal.list:3) */
 extern void gs_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
